@@ -24,6 +24,7 @@
 #include <libxml/debugXML.h>
 #include <libxml/xmlIO.h>
 #include <libxml/SAX.h>
+#include <libxml/uri.h>
 
 /* #define DEBUG_SAX */
 /* #define DEBUG_SAX_TREE */
@@ -144,7 +145,10 @@ hasExternalSubset(void *ctx)
 
 /**
  * internalSubset:
- * @ctx: the user data (XML parser context)
+ * @ctx:  the user data (XML parser context)
+ * @name:  the root element name
+ * @ExternalID:  the external ID
+ * @SystemID:  the SYSTEM ID (e.g. filename or URL)
  *
  * Callback on internal subset declaration.
  */
@@ -163,6 +167,9 @@ internalSubset(void *ctx, const xmlChar *name,
 /**
  * externalSubset:
  * @ctx: the user data (XML parser context)
+ * @name:  the root element name
+ * @ExternalID:  the external ID
+ * @SystemID:  the SYSTEM ID (e.g. filename or URL)
  *
  * Callback on external subset declaration.
  */
@@ -187,6 +194,7 @@ externalSubset(void *ctx, const xmlChar *name,
 	int oldwellFormed;
 	xmlParserInputPtr input = NULL;
 	xmlCharEncoding enc;
+	xmlCharEncoding oldcharset;
 
 	/*
 	 * Ask the Entity resolver to load the damn thing
@@ -208,6 +216,7 @@ externalSubset(void *ctx, const xmlChar *name,
 	oldinputMax = ctxt->inputMax;
 	oldinputTab = ctxt->inputTab;
 	oldwellFormed = ctxt->wellFormed;
+	oldcharset = ctxt->charset;
 
 	ctxt->inputTab = (xmlParserInputPtr *)
 	                 xmlMalloc(5 * sizeof(xmlParserInputPtr));
@@ -221,6 +230,7 @@ externalSubset(void *ctx, const xmlChar *name,
 	    ctxt->inputNr = oldinputNr;
 	    ctxt->inputMax = oldinputMax;
 	    ctxt->inputTab = oldinputTab;
+	    ctxt->charset = oldcharset;
 	    return;
 	}
 	ctxt->inputNr = 0;
@@ -263,6 +273,7 @@ externalSubset(void *ctx, const xmlChar *name,
 	ctxt->inputNr = oldinputNr;
 	ctxt->inputMax = oldinputMax;
 	ctxt->inputTab = oldinputTab;
+	ctxt->charset = oldcharset;
 	/* ctxt->wellFormed = oldwellFormed; */
     }
 }
@@ -378,11 +389,12 @@ entityDecl(void *ctx, const xmlChar *name, int type,
 /**
  * attributeDecl:
  * @ctx: the user data (XML parser context)
+ * @elem:  the name of the element
  * @fullname:  the attribute name 
  * @type:  the attribute type 
- * @publicId: The public ID of the attribute
- * @systemId: The system ID of the attribute
- * @content: the attribute value (without processing).
+ * @def:  the type of default value
+ * @defaultValue: the attribute default value
+ * @tree:  the tree of enumerated value set
  *
  * An attribute definition has been parsed
  */
@@ -428,9 +440,7 @@ attributeDecl(void *ctx, const xmlChar *elem, const xmlChar *fullname,
  * @ctx: the user data (XML parser context)
  * @name:  the element name 
  * @type:  the element type 
- * @publicId: The public ID of the element
- * @systemId: The system ID of the element
- * @content: the element value (without processing).
+ * @content: the element value tree
  *
  * An element definition has been parsed
  */
@@ -590,12 +600,29 @@ endDocument(void *ctx)
     if (ctxt->validate && ctxt->wellFormed &&
         ctxt->myDoc && ctxt->myDoc->intSubset)
 	ctxt->valid &= xmlValidateDocumentFinal(&ctxt->vctxt, ctxt->myDoc);
+
+    /*
+     * Grab the encoding if it was added on-the-fly
+     */
+    if ((ctxt->encoding != NULL) && (ctxt->myDoc != NULL) &&
+	(ctxt->myDoc->encoding == NULL)) {
+	ctxt->myDoc->encoding = ctxt->encoding;
+	ctxt->encoding = NULL;
+    }
+    if ((ctxt->inputTab[0]->encoding != NULL) && (ctxt->myDoc != NULL) &&
+	(ctxt->myDoc->encoding == NULL)) {
+	ctxt->myDoc->encoding = xmlStrdup(ctxt->inputTab[0]->encoding);
+    }
+    if ((ctxt->charset != XML_CHAR_ENCODING_NONE) && (ctxt->myDoc != NULL) &&
+	(ctxt->myDoc->charset == XML_CHAR_ENCODING_NONE)) {
+	ctxt->myDoc->charset = ctxt->charset;
+    }
 }
 
 /**
  * attribute:
  * @ctx: the user data (XML parser context)
- * @name:  The attribute name
+ * @fullname:  The attribute name, including namespace prefix
  * @value:  The attribute value
  *
  * Handle an attribute that has been read by the parser.
@@ -626,7 +653,10 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
     /*
      * Do the last stave of the attribute normalization
      */
-    nval = xmlValidNormalizeAttributeValue(ctxt->myDoc,
+    if (ctxt->html)
+	nval = NULL;
+    else
+	nval = xmlValidNormalizeAttributeValue(ctxt->myDoc,
 			       ctxt->node, fullname, value);
     if (nval != NULL)
 	value = nval;
@@ -634,9 +664,25 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
     /*
      * Check whether it's a namespace definition
      */
-    if ((ns == NULL) &&
+    if ((!ctxt->html) && (ns == NULL) &&
         (name[0] == 'x') && (name[1] == 'm') && (name[2] == 'l') &&
         (name[3] == 'n') && (name[4] == 's') && (name[5] == 0)) {
+	xmlURIPtr uri;
+
+	uri = xmlParseURI((const char *)value);
+	if (uri == NULL) {
+	    if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
+		ctxt->sax->warning(ctxt->userData, 
+		     "nmlns: %s not a valid URI\n", value);
+	} else {
+	    if (uri->scheme == NULL) {
+		if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL))
+		    ctxt->sax->warning(ctxt->userData, 
+			 "nmlns: URI %s is not absolute\n", value);
+	    }
+	    xmlFreeURI(uri);
+	}
+
 	/* a default namespace definition */
 	xmlNewNs(ctxt->node, value, NULL);
 	if (name != NULL) 
@@ -645,7 +691,8 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
 	    xmlFree(nval);
 	return;
     }
-    if ((ns != NULL) && (ns[0] == 'x') && (ns[1] == 'm') && (ns[2] == 'l') &&
+    if ((!ctxt->html) &&
+	(ns != NULL) && (ns[0] == 'x') && (ns[1] == 'm') && (ns[2] == 'l') &&
         (ns[3] == 'n') && (ns[4] == 's') && (ns[5] == 0)) {
 	/*
 	 * Validate also for namespace decls, they are attributes from
@@ -687,7 +734,7 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
 		    ret->last = tmp;
 		tmp = tmp->next;
 	    }
-	} else {
+	} else if (value != NULL) {
 	    ret->children = xmlNewDocText(ctxt->myDoc, value);
 	    ret->last = ret->children;
 	    if (ret->children != NULL)
@@ -695,7 +742,7 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
 	}
     }
 
-    if (ctxt->validate && ctxt->wellFormed &&
+    if ((!ctxt->html) && ctxt->validate && ctxt->wellFormed &&
         ctxt->myDoc && ctxt->myDoc->intSubset) {
 	
 	/*
@@ -743,7 +790,7 @@ attribute(void *ctx, const xmlChar *fullname, const xmlChar *value)
 /**
  * startElement:
  * @ctx: the user data (XML parser context)
- * @name:  The element name
+ * @fullname:  The element name, including namespace prefix
  * @atts:  An array of name/value attributes pairs, NULL terminated
  *
  * called when an opening tag has been processed.
@@ -803,6 +850,7 @@ startElement(void *ctx, const xmlChar *fullname, const xmlChar **atts)
     } else if (parent == NULL) {
         parent = ctxt->myDoc->children;
     }
+    ctxt->nodemem = -1;
 
     /*
      * We are parsing a new node.
@@ -830,15 +878,6 @@ startElement(void *ctx, const xmlChar *fullname, const xmlChar **atts)
 	}
     }
 
-    /*
-     * If it's the Document root, finish the Dtd validation and
-     * check the document root element for validity
-     */
-    if ((ctxt->validate) && (ctxt->vctxt.finishDtd == 0)) {
-	ctxt->valid &= xmlValidateDtdFinal(&ctxt->vctxt, ctxt->myDoc);
-	ctxt->valid &= xmlValidateRoot(&ctxt->vctxt, ctxt->myDoc);
-	ctxt->vctxt.finishDtd = 1;
-    }
     /*
      * process all the attributes whose name start with "xml"
      */
@@ -846,31 +885,14 @@ startElement(void *ctx, const xmlChar *fullname, const xmlChar **atts)
         i = 0;
 	att = atts[i++];
 	value = atts[i++];
-        while ((att != NULL) && (value != NULL)) {
-	    if ((att[0] == 'x') && (att[1] == 'm') && (att[2] == 'l'))
-		attribute(ctxt, att, value);
+	if (!ctxt->html) {
+	    while ((att != NULL) && (value != NULL)) {
+		if ((att[0] == 'x') && (att[1] == 'm') && (att[2] == 'l'))
+		    attribute(ctxt, att, value);
 
-	    att = atts[i++];
-	    value = atts[i++];
-	}
-    }
-
-    /*
-     * process all the other attributes
-     */
-    if (atts != NULL) {
-        i = 0;
-	att = atts[i++];
-	value = atts[i++];
-        while ((att != NULL) && (value != NULL)) {
-	    if ((att[0] != 'x') || (att[1] != 'm') || (att[2] != 'l'))
-		attribute(ctxt, att, value);
-
-	    /*
-	     * Next ones
-	     */
-	    att = atts[i++];
-	    value = atts[i++];
+		att = atts[i++];
+		value = atts[i++];
+	    }
 	}
     }
 
@@ -883,6 +905,43 @@ startElement(void *ctx, const xmlChar *fullname, const xmlChar **atts)
 	ns = xmlSearchNs(ctxt->myDoc, parent, prefix);
     xmlSetNs(ret, ns);
 
+    /*
+     * process all the other attributes
+     */
+    if (atts != NULL) {
+        i = 0;
+	att = atts[i++];
+	value = atts[i++];
+	if (ctxt->html) {
+	    while (att != NULL) {
+		attribute(ctxt, att, value);
+		att = atts[i++];
+		value = atts[i++];
+	    }
+	} else {
+	    while ((att != NULL) && (value != NULL)) {
+		if ((att[0] != 'x') || (att[1] != 'm') || (att[2] != 'l'))
+		    attribute(ctxt, att, value);
+
+		/*
+		 * Next ones
+		 */
+		att = atts[i++];
+		value = atts[i++];
+	    }
+	}
+    }
+
+    /*
+     * If it's the Document root, finish the Dtd validation and
+     * check the document root element for validity
+     */
+    if ((ctxt->validate) && (ctxt->vctxt.finishDtd == 0)) {
+	ctxt->valid &= xmlValidateDtdFinal(&ctxt->vctxt, ctxt->myDoc);
+	ctxt->valid &= xmlValidateRoot(&ctxt->vctxt, ctxt->myDoc);
+	ctxt->vctxt.finishDtd = 1;
+    }
+
     if (prefix != NULL)
 	xmlFree(prefix);
     if (name != NULL)
@@ -918,6 +977,7 @@ endElement(void *ctx, const xmlChar *name)
       node_info.node = cur;
       xmlParserAddNodeInfo(ctxt, &node_info);
     }
+    ctxt->nodemem = -1;
 
     if (ctxt->validate && ctxt->wellFormed &&
         ctxt->myDoc && ctxt->myDoc->intSubset)
@@ -994,14 +1054,62 @@ characters(void *ctx, const xmlChar *ch, int len)
 #ifdef DEBUG_SAX_TREE
     fprintf(stderr, "add chars to %s \n", ctxt->node->name);
 #endif
-    if (lastChild == NULL)
+
+    /*
+     * Here we needed an accelerator mechanism in case of very large
+     * elements. Use an attribute in the structure !!!
+     */
+    if (lastChild == NULL) {
+	/* first node, first time */
 	xmlNodeAddContentLen(ctxt->node, ch, len);
-    else {
-	if (xmlNodeIsText(lastChild))
+#ifndef XML_USE_BUFFER_CONTENT
+	if (ctxt->node->children != NULL) {
+	    ctxt->nodelen = len;
+	    ctxt->nodemem = len + 1;
+	}
+#endif
+    } else {
+	if (xmlNodeIsText(lastChild)) {
+#ifndef XML_USE_BUFFER_CONTENT
+	    /*
+	     * The whole point of maintaining nodelen and nodemem,
+	     * xmlTextConcat is too costly, i.e. compute lenght,
+	     * reallocate a new buffer, move data, append ch. Here
+	     * We try to minimaze realloc() uses and avoid copying
+	     * and recomputing lenght over and over.
+	     */
+	    if (ctxt->nodelen + len >= ctxt->nodemem) {
+		xmlChar *newbuf;
+		int size;
+
+		size = ctxt->nodemem + len;
+		size *= 2;
+                newbuf = (xmlChar *) xmlRealloc(lastChild->content,size);
+		if (newbuf == NULL) {
+		    if ((ctxt->sax != NULL) && (ctxt->sax->error != NULL))
+			ctxt->sax->error(ctxt->userData, 
+			     "SAX.characters(): out of memory\n");
+		    return;
+		}
+		ctxt->nodemem = size;
+		lastChild->content = newbuf;
+	    }
+	    memcpy(&lastChild->content[ctxt->nodelen], ch, len);
+	    ctxt->nodelen += len;
+	    lastChild->content[ctxt->nodelen] = 0;
+#else
 	    xmlTextConcat(lastChild, ch, len);
-	else {
+#endif
+	} else {
+	    /* Mixed content, first time */
 	    lastChild = xmlNewTextLen(ch, len);
 	    xmlAddChild(ctxt->node, lastChild);
+#ifndef XML_USE_BUFFER_CONTENT
+	    if (ctxt->node->children != NULL) {
+		ctxt->nodelen = len;
+		ctxt->nodemem = len + 1;
+	    }
+#endif
 	}
     }
 }
@@ -1029,7 +1137,6 @@ ignorableWhitespace(void *ctx, const xmlChar *ch, int len)
  * @ctx: the user data (XML parser context)
  * @target:  the target name
  * @data: the PI data's
- * @len: the number of xmlChar
  *
  * A processing instruction has been parsed.
  */
@@ -1102,6 +1209,7 @@ globalNamespace(void *ctx, const xmlChar *href, const xmlChar *prefix)
  *
  * Set the current element namespace.
  */
+
 void
 setNamespace(void *ctx, const xmlChar *name)
 {
@@ -1128,7 +1236,10 @@ setNamespace(void *ctx, const xmlChar *name)
  * @ctx: the user data (XML parser context)
  *
  * Get the current element namespace.
+ *
+ * Returns the xmlNsPtr or NULL if none
  */
+
 xmlNsPtr
 getNamespace(void *ctx)
 {
@@ -1149,7 +1260,10 @@ getNamespace(void *ctx)
  *
  * Check that the current element namespace is the same as the
  * one read upon parsing.
+ *
+ * Returns 1 if true 0 otherwise
  */
+
 int
 checkNamespace(void *ctx, xmlChar *namespace)
 {
