@@ -13,7 +13,7 @@
 #ifndef __XML_XLINK_H__
 #define __XML_XLINK_H__
 
-#include "tree.h"
+#include <libxml/tree.h>
 
 #ifdef __cplusplus
 #define extern "C" {
@@ -148,12 +148,13 @@ typedef void
  * There is no default xlink callbacks, if one want to get link
  * recognition activated, those call backs must be provided before parsing.
  */
-typedef struct xlinkHandler {
+typedef struct _xlinkHandler xlinkHandler;
+typedef xlinkHandler *xlinkHandlerPtr;
+struct _xlinkHandler {
     xlinkSimpleLinkFunk simple;
     xlinkExtendedLinkFunk extended;
     xlinkExtendedLinkSetFunk set;
-} xlinkHandler;
-typedef xlinkHandler *xlinkHandlerPtr;
+};
 
 /**
  * the default detection routine, can be overriden, they call the default
